////////////////////////////////////////////////////////////
//  Puts atoms randomly in a box, subject to distance     //
//  cutoffs                                               //
//                                                        //
//  JKC, October 2009                                     //
////////////////////////////////////////////////////////////

#include <iostream>
#include <fstream>
#include <cmath>
#include <stdlib.h>

using namespace std;

double randone()
{
    // returns a number uniformly distributed between 0 and 1
    double rr = rand();
    return rr/static_cast<double>(RAND_MAX);
}

int rounding (double x)
{
    // rounds a number to the nearest integer
    int intx = static_cast<int>(x); // truncates
    double dx = x - intx;
    if(dx<0.5){return intx;}
    else{return (intx+1);}
}

int main(int argc, char **argv)
{
    srand(time(NULL));

    if(argc<10)
    {
	cerr << "Usage: " << argv[0] << " <P-O cutoff (A)> <M-O cutoff (A)> <X-X cutoff (A)> <natoms> <density (g/cm3)> <density correction (can be 0.95)> <P2O5 mol %> <Na2O mol %> <CaO mol %>" << endl;
	return 1;
    }

    int natoms_intended = atoi(argv[4]);

    double mol_P2O5 = atof(argv[7]);
    double mol_Na2O = atof(argv[8]);
    double mol_CaO = atof(argv[9]);

    // sanity check on percentages
    if(fabs(mol_P2O5+mol_Na2O+mol_CaO-100.0)>0.03)
    {
	cerr << "Percentages do not add up to 100.0! " << mol_P2O5+mol_Na2O+mol_CaO << endl;
	return 1;
    }

    // compute the total number of formula units

    int nunits = rounding(100.0*natoms_intended/(mol_P2O5*7.0 + mol_Na2O*3.0 + mol_CaO*2.0));
    
    // compute the number of formula units of each molecule

    int nunits_P2O5 = rounding(mol_P2O5*nunits/100.0);
    int nunits_Na2O = rounding(mol_Na2O*nunits/100.0);
    int nunits_CaO = rounding(mol_CaO*nunits/100.0);

    // numbers of atoms

    int nO = 5*nunits_P2O5 + nunits_Na2O + nunits_CaO;
    int nP = 2*nunits_P2O5;
    int nNa = 2*nunits_Na2O;
    int nCa = nunits_CaO;
    int natoms = nO+nP+nNa+nCa;

    // output true composition

    int nunits_true = nunits_P2O5+nunits_Na2O+nunits_CaO;

    cerr << "Simulating " << natoms << " atoms" << endl;
    cerr << 100.0*double(nunits_P2O5)/(double)nunits_true << " mol% P2O5" << endl;
    cerr << 100.0*double(nunits_Na2O)/(double)nunits_true << " mol% Na2O" << endl;
    cerr << 100.0*double(nunits_CaO)/(double)nunits_true << " mol% CaO" << endl;
     
    // define mass
    double mass = (nP*30.97376 + nNa*22.98977 + nCa*40.078 + nO*15.9994)*1.66053886e-24; // in grams

    cerr << "mass: " << mass << endl;

    // required density
    double density = atof(argv[5])*atof(argv[6]); // g/cm^3

    // hence the volume
    double volume = mass/density; // in cm^3

    // define the box size
    double box = pow(volume,1.0/3.0)*1e8; // angstroms
    
    cerr << box << endl;

    // put the oxygen atoms in

    double cutoff_OO = atof(argv[3])/box;

    double Opos[nO][3]; // oxygen atom positions
    int ACCEPT;
    double trialpos[3],dx,dy,dz,dr2;
    int accepted=0;
    do{
	ACCEPT=1;
	// choose trial position
	trialpos[0] = randone();
	trialpos[1] = randone();
	trialpos[2] = randone();

	// check interatomic distances
	for(int j=0; j<accepted && ACCEPT==1; j++)
	{
	    dx = Opos[j][0] - trialpos[0];
	    dy = Opos[j][1] - trialpos[1];
	    dz = Opos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_OO*cutoff_OO){ACCEPT=0;}
	}
	if(ACCEPT==1)
	{
	    // then accept the atom position
	    Opos[accepted][0] = trialpos[0];
	    Opos[accepted][1] = trialpos[1];
	    Opos[accepted][2] = trialpos[2];
	    accepted++;
            if(accepted%100==0){cerr << accepted << " ";}
	}

	
    }while(ACCEPT==0 || (ACCEPT==1 && accepted<nO)); 
    
    // put the phosphorous atoms in 

    double cutoff_PO = atof(argv[1])/box;
    double cutoff_PP = atof(argv[3])/box;

    double Ppos[nP][3]; // silicon atom positions
    accepted=0;
   
    do{
	ACCEPT=1;
	// choose trial position
	trialpos[0] = randone();
	trialpos[1] = randone();
	trialpos[2] = randone();
	
	// check P-O interatomic distances
	for(int j=0; j<nO && ACCEPT==1; j++)
	{
	    dx = Opos[j][0] - trialpos[0];
	    dy = Opos[j][1] - trialpos[1];
	    dz = Opos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_PO*cutoff_PO){ACCEPT=0;}
	}

	// check P-P interatomic distances
	for(int j=0; j<accepted && ACCEPT==1; j++)
	{
	    dx = Ppos[j][0] - trialpos[0];
	    dy = Ppos[j][1] - trialpos[1];
	    dz = Ppos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_PP*cutoff_PP){ACCEPT=0;}
	}

	if(ACCEPT==1)
	{
	    // then accept the atom position
	    Ppos[accepted][0] = trialpos[0];
	    Ppos[accepted][1] = trialpos[1];
	    Ppos[accepted][2] = trialpos[2];
	    accepted++;
	}
	
    }while(ACCEPT==0 || (ACCEPT==1 && accepted<nP)); 


    // put sodium atoms in

    double cutoff_NaO = atof(argv[2])/box;
    double cutoff_NaP = atof(argv[3])/box;
    double cutoff_NaNa = atof(argv[3])/box;

    double Napos[nNa][3]; // sodium atom positions
    accepted=0;
   
    do{
	ACCEPT=1;
	// choose trial position
	trialpos[0] = randone();
	trialpos[1] = randone();
	trialpos[2] = randone();
	
	// check Na-O interatomic distances
	for(int j=0; j<nO && ACCEPT==1; j++)
	{
	    dx = Opos[j][0] - trialpos[0];
	    dy = Opos[j][1] - trialpos[1];
	    dz = Opos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_NaO*cutoff_NaO){ACCEPT=0;}
	}

	// check Na-P interatomic distances
	for(int j=0; j<nP && ACCEPT==1; j++)
	{
	    dx = Ppos[j][0] - trialpos[0];
	    dy = Ppos[j][1] - trialpos[1];
	    dz = Ppos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_NaP*cutoff_NaP){ACCEPT=0;}
	}

	// check Na-Na interatomic distances
	for(int j=0; j<accepted && ACCEPT==1; j++)
	{
	    dx = Napos[j][0] - trialpos[0];
	    dy = Napos[j][1] - trialpos[1];
	    dz = Napos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_NaNa*cutoff_NaNa){ACCEPT=0;}
	}

	if(ACCEPT==1)
	{
	    // then accept the atom position
	    Napos[accepted][0] = trialpos[0];
	    Napos[accepted][1] = trialpos[1];
	    Napos[accepted][2] = trialpos[2];
	    accepted++;
	}
	
    }while(ACCEPT==0 || (ACCEPT==1 && accepted<nNa)); 


    // put the calcium atoms in

    double cutoff_CaO = atof(argv[2])/box;
    double cutoff_CaP = atof(argv[3])/box;
    double cutoff_CaNa = atof(argv[3])/box;
    double cutoff_CaCa = atof(argv[3])/box;

    double Capos[nCa][3]; // calcium atom positions
    accepted=0;
    
    do{
	ACCEPT=1;
	// choose trial position
	trialpos[0] = randone();
	trialpos[1] = randone();
	trialpos[2] = randone();
	
	// check Ca-O interatomic distances
	for(int j=0; j<nO && ACCEPT==1; j++)
	{
	    dx = Opos[j][0] - trialpos[0];
	    dy = Opos[j][1] - trialpos[1];
	    dz = Opos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CaO*cutoff_CaO){ACCEPT=0;}
	}

	// check Ca-P interatomic distances
	for(int j=0; j<nP && ACCEPT==1; j++)
	{
	    dx = Ppos[j][0] - trialpos[0];
	    dy = Ppos[j][1] - trialpos[1];
	    dz = Ppos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CaP*cutoff_CaP){ACCEPT=0;}
	}

	// check Ca-Na interatomic distances
	for(int j=0; j<nNa && ACCEPT==1; j++)
	{
	    dx = Napos[j][0] - trialpos[0];
	    dy = Napos[j][1] - trialpos[1];
	    dz = Napos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CaNa*cutoff_CaNa){ACCEPT=0;}
	}

	// check Ca-Ca interatomic distances
	for(int j=0; j<accepted && ACCEPT==1; j++)
	{
	    dx = Capos[j][0] - trialpos[0];
	    dy = Capos[j][1] - trialpos[1];
	    dz = Capos[j][2] - trialpos[2];
	    
	    // correct for PBCs
	    while(dx>0.5){dx-=1.0;}
	    while(dy>0.5){dy-=1.0;}
	    while(dz>0.5){dz-=1.0;}
	    while(dx<-0.5){dx+=1.0;}
	    while(dy<-0.5){dy+=1.0;}
	    while(dz<-0.5){dz+=1.0;}
	
	    dr2 = dx*dx + dy*dy + dz*dz;
	    if(dr2<cutoff_CaCa*cutoff_CaCa){ACCEPT=0;}
	}

	if(ACCEPT==1)
	{
	    // then accept the atom position
	    Capos[accepted][0] = trialpos[0];
	    Capos[accepted][1] = trialpos[1];
	    Capos[accepted][2] = trialpos[2];
	    accepted++;
	}
	
    }while(ACCEPT==0 || (ACCEPT==1 && accepted<nCa)); 

    // output atom positions in xyz format
// with the cell parameter at the front
    cout << "LAMMPS atom data file" << endl << endl;
    
    cout << "0.0000 " << box << " xlo xhi" << endl;
    cout << "0.0000 " << box << " ylo yhi" << endl;
    cout << "0.0000 " << box << " zlo zhi" << endl << endl;
    
    cout << "4 atom types" << endl << endl;

    cout << natoms << " atoms" << endl << endl;

    cout << "Atoms" << endl;
    cout << " Atom ID, Atom type, q, X, Y, Z" << endl;
for (int i=0; i<nP; i++)
    {
        cout << " P " << " " << Ppos[i][0]*box << " " << Ppos[i][1]*box << " " << Ppos[i][2]*box << endl;
    }
for (int i=0; i<nO; i++)
    {
        cout << " O " << " " << Opos[i][0]*box << " " << Opos[i][1]*box << " " << Opos[i][2]*box << endl;
    }
for (int i=0; i<nNa; i++)
    {
        cout << " Na " << " " << Napos[i][0]*box << " " << Napos[i][1]*box << " " << Napos[i][2]*box << endl;
    }

for (int i=0; i<nCa; i++)
    {
        cout << " Ca " << " " << Capos[i][0]*box << " " << Capos[i][1]*box << " " << Capos[i][2]*box << endl;
    }


  
    // output guidance for field file
    ofstream fout;
    fout.open("notesFIELD");
    fout << nO << " oxygen" << endl;
    fout << nP << " phosphorous" << endl;
    fout << nNa << " sodium" << endl;
    fout << nCa << " calcium" << endl;
    fout.close();

    return 0;
}
